from typing import Optional

from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Episode(Base):
    def __init__(self, tv_id: int, season_id: int, episode_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.tv_id = tv_id
        self.season_id = season_id
        self.episode_id = episode_id

    async def details(self, *, append_to_response: Optional[str] = None) -> schemas.Episode:
        """Get the TV season details by id.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-details
        """
        data = await self.request(f"tv/{self.tv_id}/season/{self.season_id}/episode/{self.episode_id}", append_to_response=append_to_response)
        return utils.as_dataclass(schemas.Episode, data)

    async def credits(self) -> schemas.Credits:
        """Get the credits for TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-credits
        """
        data = await self.request(f"tv/{self.tv_id}/season/{self.season_id}/episode/{self.episode_id}/credits")
        return utils.as_dataclass(schemas.Credits, data)

    async def external_ids(self) -> schemas.ExternalIDs:
        """Get the external ids for a TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-external-ids
        """
        data = await self.request(f"tv/{self.tv_id}/season/{self.season_id}/episode/{self.episode_id}/external_ids")
        return utils.as_dataclass(schemas.ExternalIDs, data)

    async def images(self) -> schemas.Images:
        """Get the images that belong to a TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-images
        """
        data = await self.request(f"tv/{self.tv_id}/season/{self.season_id}/episode/{self.episode_id}/images")
        return utils.as_dataclass(schemas.Images, data)

    async def translations(self) -> schemas.Translations:
        """Get the credits for TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-translations
        """
        data = await self.request(f"tv/{self.tv_id}/season/{self.season_id}/episode/{self.episode_id}/translations")
        return utils.as_dataclass(schemas.Translations, data)

    async def videos(self) -> schemas.Videos:
        """Get the videos that have been added to a TV season.

        See more: https://developers.themoviedb.org/3/tv-seasons/get-tv-season-videos
        """
        data = await self.request(f"tv/{self.tv_id}/season/{self.season_id}/episode/{self.episode_id}/videos")
        return utils.as_dataclass(schemas.Videos, data)



from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._partial import PartialCollection, PartialMovie
from themoviedb.schemas._result import ResultWithID


@dataclass
class ContentRating:
    iso_3166_1: Optional[str] = None
    rating: Optional[str] = None


@dataclass
class ContentRatings(ResultWithID):
    results: Optional[List[ContentRating]] = None



from themoviedb import schemas, utils
from themoviedb.routes_sync._base import Base


class Credit(Base):
    def __init__(self, credit_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.credit_id = credit_id

    def details(self) -> schemas.Credit:
        """Get a movie or TV credit details by id.

        See more: https://developers.themoviedb.org/3/credits/get-credit-details
        """
        data = self.request(f"credit/{self.credit_id}")
        return utils.as_dataclass(schemas.Credit, data)



from themoviedb.routes_async._base import Base
from themoviedb.routes_async.authentication import Authentication
from themoviedb.routes_async.certifications import Certifications
from themoviedb.routes_async.credit import Credit
from themoviedb.routes_async.companies import Company
from themoviedb.routes_async.collections import Collection
from themoviedb.routes_async.discover import Discover
from themoviedb.routes_async.episode_groups import EpisodeGroup
from themoviedb.routes_async.episodes import Episode
from themoviedb.routes_async.find import Find
from themoviedb.routes_async.genres import Genres
from themoviedb.routes_async.keywords import Keyword
from themoviedb.routes_async.movies import Movie, Movies
from themoviedb.routes_async.people import People, Person
from themoviedb.routes_async.network import Network
from themoviedb.routes_async.guest import Guest
from themoviedb.routes_async.reviews import Review
from themoviedb.routes_async.search import Search
from themoviedb.routes_async.seasons import Season
from themoviedb.routes_async.trending import Trending
from themoviedb.routes_async.tv import TV, TVs
from themoviedb.routes_async.watch_providers import WatchProviders



from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Company(Base):
    def __init__(self, company_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.company_id = company_id

    async def details(self) -> schemas.Company:
        """Get a companies details by id.

        See more: https://developers.themoviedb.org/3/companies/get-company-details
        """
        data = await self.request(f"company/{self.company_id}")
        return utils.as_dataclass(schemas.Company, data)

    async def alternative_names(self) -> schemas.AlternativeNames:
        """Get the alternative names of a company.

        See more: https://developers.themoviedb.org/3/companies/get-company-alternative_names
        """
        data = await self.request(f"company/{self.company_id}/alternative_names")
        return utils.as_dataclass(schemas.AlternativeNames, data)

    async def images(self) -> schemas.Images:
        """Get a companies logos by id.

        See more: https://developers.themoviedb.org/3/companies/get-company-images
        """
        data = await self.request(f"company/{self.company_id}/images")
        return utils.as_dataclass(schemas.Images, data)



from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Certifications(Base):

    async def movie(self) -> schemas.Certifications:
        """Get an up to date list of the officially supported movie certifications on TMDB.

        See more: https://developers.themoviedb.org/3/certifications/get-movie-certifications
        """
        data = await self.request("certification/movie/list")
        return utils.as_dataclass(schemas.Certifications, data)

    async def tv(self) -> schemas.Certifications:
        """Get an up to date list of the officially supported TV show certifications on TMDB.

        See more: https://developers.themoviedb.org/3/certifications/get-tv-certifications
        """
        data = await self.request("certification/tv/list")
        return utils.as_dataclass(schemas.Certifications, data)



from dataclasses import asdict
from datetime import datetime, date
from enum import Enum
from typing import Type, TypeVar

from dacite import Config, from_dict

T = TypeVar("T")
DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"

config = Config(
    cast=[Enum],
    type_hooks={
        datetime: lambda x: datetime.fromisoformat(x.rstrip("Z")) if x else None,
        date: lambda x: datetime.fromisoformat(x.rstrip("Z")).date() if x else None,
    },
)

dict_factory = lambda data: dict(
    (
        k,
        v.value
        if isinstance(v, Enum)
        else f"{v.strftime(DATETIME_FORMAT)[:-3]}Z"
        if isinstance(v, datetime)
        else v.isoformat()
        if isinstance(v, date)
        else v,
    )
    for k, v in data
)


def as_dataclass(data_class: Type[T], data: dict) -> T:
    return from_dict(data_class, data, config=config)


def as_dict(obj):
    return asdict(obj, dict_factory=dict_factory)



from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from themoviedb.schemas._result import ResultWithID


@dataclass
class Video:
    id: str
    iso_639_1: Optional[str] = None
    iso_3166_1: Optional[str] = None
    name: Optional[str] = None
    key: Optional[str] = None
    site: Optional[str] = None
    size: Optional[int] = None
    type: Optional[str] = None
    official: Optional[bool] = None
    published_at: Optional[datetime] = None

    def __str__(self) -> str:
        return self.name


@dataclass
class Videos(ResultWithID):
    results: Optional[List[Video]] = None



from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Collection(Base):
    def __init__(self, collection_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.collection_id = collection_id

    async def details(self) -> schemas.Collection:
        """Get collection details by id.

        See more: https://developers.themoviedb.org/3/collections/get-collection-details
        """
        data = await self.request(f"collection/{self.collection_id}")
        return utils.as_dataclass(schemas.Collection, data)

    async def images(self) -> schemas.Images:
        """Get the images for a collection by id.

        See more: https://developers.themoviedb.org/3/collections/get-collection-images
        """
        data = await self.request(f"collection/{self.collection_id}/images")
        return utils.as_dataclass(schemas.Images, data)

    async def translations(self) -> schemas.Translations:
        """Get the list translations for a collection by id.

        See more: https://developers.themoviedb.org/3/collections/get-collection-translations
        """
        data = await self.request(f"collection/{self.collection_id}/translations")
        return utils.as_dataclass(schemas.Translations, data)



from enum import Enum


class CreditType(str, Enum):
    cast = "cast"
    crew = "crew"


class EpisodeGroupType(Enum):
    original_air_date = 1
    absolute = 2
    dvd = 3
    digital = 4
    story_arc = 5
    production = 6
    tv = 7

    __str_map__ = {
        1: "Original air date",
        2: "Absolute",
        3: "DVD",
        4: "Digital",
        5: "Story arc",
        6: "Production",
        7: "TV",
    }

    def __str__(self) -> str:
        return self.__str_map__[self.value]


class ImageType(str, Enum):
    backdrop = "backdrop"
    logo = "logo"
    poster = "poster"
    profile = "profile"
    still = "still"


class MediaType(str, Enum):
    movie = "movie"
    person = "person"
    tv = "tv"


class SizeType(str, Enum):
    w45 = "w45"
    w92 = "w92"
    w154 = "w154"
    w185 = "w185"
    w300 = "w300"
    w342 = "w342"
    w500 = "w500"
    h632 = "h632"
    w780 = "w780"
    w1280 = "w1280"
    original = "original"



from dataclasses import dataclass
from datetime import date
from typing import List, Optional, Type, TypeVar

T = TypeVar("T")


@dataclass
class Dates:
    maximum: Optional[date] = None
    minimum: Optional[date] = None


@dataclass
class Result:
    results: Optional[List[Type[T]]] = None

    def __bool__(self) -> bool:
        return len(self.results) > 0

    def __iter__(self) -> iter:
        return iter(self.results)

    def __getitem__(self, index: int) -> T:
        return self.results[index]

    def __len__(self) -> int:
        return len(self.results)


@dataclass
class ResultWithID(Result):
    id: Optional[int] = None


@dataclass
class ResultWithPage(ResultWithID):
    page: Optional[int] = None
    dates: Optional[Dates] = None
    total_pages: Optional[int] = None
    total_results: Optional[int] = None



from typing import Optional

from themoviedb import schemas, utils
from themoviedb.routes_sync._base import Base


class Search(Base):

    def companies(self, query: str, *, page: int = 1) -> schemas.Companies:
        """Search for companies.

        See more: https://developers.themoviedb.org/3/search/search-companies
        """
        data = self.request("search/company", query=query, page=page)
        return utils.as_dataclass(schemas.Companies, data)

    def collections(self, query: str, *, page: int = 1) -> schemas.Collections:
        """Search for collections.

        See more: https://developers.themoviedb.org/3/search/search-collections
        """
        data = self.request("search/collection", query=query, page=page)
        return utils.as_dataclass(schemas.Collections, data)

    def keywords(self, query: str, *, page: int = 1) -> schemas.Keywords:
        """Search for keywords.

        See more: https://developers.themoviedb.org/3/search/search-keywords
        """
        data = self.request("search/keyword", query=query, page=page)
        return utils.as_dataclass(schemas.Keywords, data)

    def movies(
        self,
        query: str,
        *,
        page: int = 1,
        include_adult: bool = False,
        year: Optional[int] = None,
        primary_release_year: Optional[int] = None,
    ) -> schemas.Movies:
        """Search for movies.

        See more: https://developers.themoviedb.org/3/search/search-movies
        """
        data = self.request(
            "search/movie",
            query=query,
            page=page,
            include_adult=include_adult,
            year=year,
            primary_release_year=primary_release_year,
        )
        return utils.as_dataclass(schemas.Movies, data)

    def multi(self, query: str, *, page: int = 1, include_adult: bool = False) -> schemas.Multis:
        """Search multiple models in a single request. Multi search currently
        supports searching for movies, tv shows and people in a single request.

        See more: https://developers.themoviedb.org/3/search/multi-search
        """
        data = self.request("search/multi", query=query, page=page, include_adult=include_adult)
        return utils.as_dataclass(schemas.Multis, data)

    def people(self, query: str, *, page: int = 1, include_adult: bool = False) -> schemas.People:
        """Search for people.

        See more: https://developers.themoviedb.org/3/search/search-people
        """
        data = self.request("search/person", query=query, page=page, include_adult=include_adult)
        return utils.as_dataclass(schemas.People, data)

    def tv(
        self,
        query: str,
        *,
        page: int = 1,
        include_adult: bool = False,
        first_air_date_year: Optional[int] = None,
    ) -> schemas.TVs:
        """Search for a TV show.

        See more: https://developers.themoviedb.org/3/search/search-tv-shows
        """
        data = self.request(
            "search/tv",
            query=query,
            page=page,
            include_adult=include_adult,
            first_air_date_year=first_air_date_year,
        )
        return utils.as_dataclass(schemas.TVs, data)



from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._enums import EpisodeGroupType
from themoviedb.schemas._partial import PartialEpisode, PartialNetwork
from themoviedb.schemas._result import ResultWithID


@dataclass
class Group:
    id: Optional[str] = None
    name: Optional[str] = None
    order: Optional[int] = None
    episodes: Optional[List[PartialEpisode]] = None
    locked: Optional[bool] = None


@dataclass
class EpisodeGroup:
    id: Optional[str] = None
    description: Optional[str] = None
    episode_count: Optional[int] = None
    group_count: Optional[int] = None
    groups: Optional[List[Group]] = None
    name: Optional[str] = None
    network: Optional[PartialNetwork] = None
    type: Optional[EpisodeGroupType] = None


@dataclass
class EpisodeGroups(ResultWithID):
    results: Optional[List[EpisodeGroup]] = None



from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Authentication(Base):

    async def create_guest_session(self) -> schemas.GuestAuthentication:
        """This method will let you create a new guest session.
        Guest sessions are a type of session that will let a user rate movies
        and TV shows but not require them to have a TMDB user account.

        See more: https://developers.themoviedb.org/3/authentication/create-guest-session
        """
        data = await self.request("authentication/guest_session/new")
        return utils.as_dataclass(schemas.GuestAuthentication, data)

    async def create_token(self) -> schemas.TokenAuthentication:
        """Create a temporary request token that can be used to validate a TMDB user login.

        See more: https://developers.themoviedb.org/3/authentication/create-request-token
        """
        data = await self.request("authentication/token/new")
        return utils.as_dataclass(schemas.TokenAuthentication, data)
    
    async def create_session(self, request_token: str) -> schemas.Session:
        """You can use this method to create a fully valid session ID once a user has
        validated the request token.

        See more: https://developers.themoviedb.org/3/authentication/create-session
        """
        data = await self.request(
            "authentication/session/new",
            method="POST",
            json={"request_token": request_token},
        )
        return utils.as_dataclass(schemas.Session, data)
    
    async def create_session_with_login(self, username: str, password: str, request_token: str) -> schemas.Session:
        """This method allows an application to validate a request token by entering
        a username and password.

        See more: https://developers.themoviedb.org/3/authentication/validate-request-token
        """
        data = await self.request(
            "authentication/token/validate_with_login",
            method="POST",
            json={"request_token": request_token, "username": username, "password": password},
        )
        return utils.as_dataclass(schemas.TokenAuthentication, data)
    
    async def delete_session(self, session_id: str) -> schemas.Response:
        """If you would like to delete (or "logout") from a session,
        call this method with a valid session ID.

        See more: https://developers.themoviedb.org/3/authentication/delete-session
        """
        data = await self.request(
            "authentication/session",
            method="DELETE",
            json={"session_id": session_id},
        )
        return utils.as_dataclass(schemas.Response, data)



from themoviedb.routes_sync._base import Base
from themoviedb.routes_sync.authentication import Authentication
from themoviedb.routes_sync.certifications import Certifications
from themoviedb.routes_sync.credit import Credit
from themoviedb.routes_sync.companies import Company
from themoviedb.routes_sync.collections import Collection
from themoviedb.routes_sync.discover import Discover
from themoviedb.routes_sync.episode_groups import EpisodeGroup
from themoviedb.routes_sync.episodes import Episode
from themoviedb.routes_sync.find import Find
from themoviedb.routes_sync.genres import Genres
from themoviedb.routes_sync.keywords import Keyword
from themoviedb.routes_sync.movies import Movie, Movies
from themoviedb.routes_sync.people import People, Person
from themoviedb.routes_sync.network import Network
from themoviedb.routes_sync.guest import Guest
from themoviedb.routes_sync.reviews import Review
from themoviedb.routes_sync.search import Search
from themoviedb.routes_sync.seasons import Season
from themoviedb.routes_sync.trending import Trending
from themoviedb.routes_sync.tv import TV, TVs
from themoviedb.routes_sync.watch_providers import WatchProviders



from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Keyword(Base):
    def __init__(self, keyword_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.keyword_id = keyword_id

    async def details(self) -> schemas.Keyword:
        """Get a keyword details by id.

        See more: https://developers.themoviedb.org/3/keywords/get-keyword-details
        """
        data = await self.request(f"keyword/{self.keyword_id}")
        return utils.as_dataclass(schemas.Keyword, data)

    async def movies(self, *, page: int = 1, include_adult: bool = False) -> schemas.Movies:
        """Get the movies that belong to a keyword.

        See more: https://developers.themoviedb.org/3/keywords/get-movies-by-keyword
        """
        data = await self.request(f"keyword/{self.keyword_id}/movies", page=page, include_adult=include_adult)
        return utils.as_dataclass(schemas.Movies, data)



from themoviedb import schemas, utils
from themoviedb.routes_sync._base import Base


class Network(Base):
    def __init__(self, network_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.network_id = network_id

    def details(self) -> schemas.Network:
        """Get the details of a network.

        See more: https://developers.themoviedb.org/3/networks/get-network-details
        """
        data = self.request(f"network/{self.network_id}")
        return utils.as_dataclass(schemas.Network, data)

    def alternative_names(self) -> schemas.AlternativeNames:
        """Get the alternative names of a network.

        See more: https://developers.themoviedb.org/3/networks/get-network-alternative-names
        """
        data = self.request(f"network/{self.network_id}/alternative_names")
        return utils.as_dataclass(schemas.AlternativeNames, data)

    def images(self) -> schemas.Images:
        """Get the TV network logos by id.

        See more: https://developers.themoviedb.org/3/networks/get-network-images
        """
        data = self.request(f"network/{self.network_id}/images")
        return utils.as_dataclass(schemas.Images, data)



from themoviedb.schemas._enums import (
    CreditType,
    EpisodeGroupType,
    ImageType,
    MediaType,
    SizeType,
)
from themoviedb.schemas._partial import (
    PartialCollection,
    PartialCompany,
    PartialKeyword,
    PartialMedia,
    PartialMovie,
    PartialPerson,
    PartialTV,
)
from themoviedb.schemas._result import Dates, Result, ResultWithID, ResultWithPage
from themoviedb.schemas.alternative_names import AlternativeName, AlternativeNames
from themoviedb.schemas.alternative_titles import AlternativeTitle, AlternativeTitles
from themoviedb.schemas.authentication import (
    Authentication,
    GuestAuthentication,
    Response,
    Session,
    TokenAuthentication,
)
from themoviedb.schemas.certifications import Certification, Certifications
from themoviedb.schemas.changes import Change
from themoviedb.schemas.collections import Collection, Collections
from themoviedb.schemas.companies import Companies, Company
from themoviedb.schemas.content_ratings import ContentRating, ContentRatings
from themoviedb.schemas.credit import Credit
from themoviedb.schemas.credits import (
    Cast,
    CastCombined,
    CastMovie,
    CastTV,
    Credits,
    CreditsCombined,
    CreditsMovie,
    CreditsTV,
    Crew,
    CrewCombined,
    CrewMovie,
    CrewTV,
)
from themoviedb.schemas.episode_groups import EpisodeGroup, EpisodeGroups
from themoviedb.schemas.episodes import Episode
from themoviedb.schemas.external_ids import ExternalIDs
from themoviedb.schemas.genres import Genre, Genres
from themoviedb.schemas.images import Image, Images, TaggedImage, TaggedImages
from themoviedb.schemas.keywords import Keyword, Keywords
from themoviedb.schemas.list import ItemList, ItemsList
from themoviedb.schemas.movies import Movie, Movies
from themoviedb.schemas.multi import Multi, MultiResults, Multis
from themoviedb.schemas.networks import Network
from themoviedb.schemas.people import People, Person
from themoviedb.schemas.rated import (
    RatedEpisode,
    RatedEpisodes,
    RatedMovie,
    RatedMovies,
    RatedTV,
    RatedTVs,
)
from themoviedb.schemas.regions import Region, Regions
from themoviedb.schemas.release_date import ReleaseDate, ReleaseDates
from themoviedb.schemas.reviews import Review, Reviews
from themoviedb.schemas.seasons import Season
from themoviedb.schemas.translations import Translation, Translations
from themoviedb.schemas.tv import TV, Episodes, TVs
from themoviedb.schemas.videos import Video, Videos
from themoviedb.schemas.watch_providers import (
    WatchProvider,
    WatchProviderData,
    WatchProviders,
    WatchProvidersData,
)



from datetime import datetime
from dataclasses import dataclass
from typing import Dict, List, Optional, Union


@dataclass
class Response:
    success: Optional[bool] = None


@dataclass
class Authentication(Response):
    expires_at: Optional[str] = None

    @property
    def valid_until(self) -> Optional[datetime]:
        try:
            return datetime.strptime(self.expires_at, "%Y-%m-%d %H:%M:%S UTC")
        except Exception as e:
            return None


@dataclass
class GuestAuthentication(Authentication):
    guest_session_id: Optional[str] = None


@dataclass
class TokenAuthentication(Authentication):
    request_token: Optional[str] = None


@dataclass
class Session(Response):
    session_id: Optional[str] = None



from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from themoviedb.schemas._enums import ImageType, MediaType, SizeType
from themoviedb.schemas._partial import PartialMedia
from themoviedb.schemas._result import ResultWithPage


@dataclass
class Image:
    id: Optional[str] = None
    aspect_ratio: Optional[float] = None
    file_path: Optional[str] = None
    height: Optional[int] = None
    file_type: Optional[str] = None
    iso_639_1: Optional[str] = None
    vote_average: Optional[float] = None
    vote_count: Optional[int] = None
    width: Optional[int] = None

    def file_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.file_path}" if self.file_path else None


@dataclass
class TaggedImage(Image):
    image_type: Optional[ImageType] = None
    media: Optional[PartialMedia] = None
    media_type: Optional[MediaType] = None


@dataclass
class Images:
    id: Optional[int] = None
    backdrops: Optional[List[Image]] = None
    logos: Optional[List[Image]] = None
    posters: Optional[List[Image]] = None
    profiles: Optional[List[Image]] = None
    stills: Optional[List[Image]] = None


@dataclass
class TaggedImages(ResultWithPage):
    results: Optional[List[TaggedImage]] = None



from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._enums import MediaType
from themoviedb.schemas._partial import (
    PartialEpisode,
    PartialMovie,
    PartialPerson,
    PartialSeason,
    PartialTV,
)
from themoviedb.schemas._result import ResultWithPage


@dataclass
class Multi(PartialMovie, PartialPerson, PartialTV):

    def __str__(self) -> str:
        return self.name or self.title or self.original_name or self.original_title

    def is_movie(self) -> str:
        return self.media_type == MediaType.movie

    def is_person(self) -> str:
        return self.media_type == MediaType.person

    def is_tv(self) -> bool:
        return self.media_type == MediaType.tv


@dataclass
class Multis(ResultWithPage):
    results: Optional[List[Multi]] = None


@dataclass
class MultiResults:
    movie_results: Optional[List[PartialMovie]] = None
    person_results: Optional[List[PartialPerson]] = None
    tv_results: Optional[List[PartialTV]] = None
    tv_episode_results: Optional[List[PartialEpisode]] = None
    tv_season_results: Optional[List[PartialSeason]] = None



from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class EpisodeGroup(Base):
    def __init__(self, episode_group_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.episode_group_id = episode_group_id

    async def details(self) -> schemas.EpisodeGroup:
        """Get the details of a TV episode group.

        See more: https://developers.themoviedb.org/3/tv-episode-groups/get-tv-episode-group-details
        """
        data = await self.request(f"tv/episode_group/{self.episode_group_id}")
        return utils.as_dataclass(schemas.EpisodeGroup, data)



#!/usr/bin/env python

from setuptools import setup, find_packages

with open("README.rst", "r", encoding="utf-8") as readme_file:
    readme = readme_file.read()

requirements = [
    "aiohttp==3.8.0",
    "dacite==1.8.0",
    "requests==2.28.2",
]

test_requirements = [
    "pytest==3",
    "pytest-asyncio==0.20.3",
]

setup(
    name="themoviedb",
    description="A modern and easy to use API wrapper for The Movie Database (TMDb) API v3 written in Python",
    long_description=readme,
    author="Leandro César",
    author_email="ccleandroc@gmail.com",
    url="https://github.com/leandcesar/themoviedb",
    version="0.3.3",
    license="MIT",
    python_requires=">=3.7",
    packages=find_packages(include=["themoviedb", "themoviedb.*"]),
    include_package_data=True,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    keywords=[
        "tmdb",
        "tmdb3",
        "aiotmdb",
        "aiotmdb3",
        "themoviedb",
        "themoviedb3",
        "sync",
        "async",
        "await",
        "aio",
        "movie",
        "movies",
        "tv",
        "tv show",
        "tv shows",
        "api",
        "wrapper",
    ],
    zip_safe=False,
    install_requires=requirements,
    tests_require=test_requirements,
    test_suite="tests",
)



from dataclasses import dataclass
from typing import Optional

from themoviedb.schemas._enums import MediaType


@dataclass
class ExternalIDs:
    id: Optional[int] = None
    facebook_id: Optional[str] = None
    freebase_id: Optional[str] = None
    freebase_mid: Optional[str] = None
    imdb_id: Optional[str] = None
    instagram_id: Optional[str] = None
    tvdb_id: Optional[int] = None
    tvrage_id: Optional[int] = None
    twitter_id: Optional[str] = None
    tiktok_id: Optional[str] = None
    youtube_id: Optional[str] = None
    wikidata_id: Optional[str] = None

    @property
    def imdb_url(self) -> Optional[str]:
        if self.imdb_id and self.imdb_id.startswith("nm"):
            return f"https://www.imdb.com/name/{self.imdb_id}"
        if self.imdb_id and self.imdb_id.startswith("tt"):
            return f"https://www.imdb.com/title/{self.imdb_id}"
        return None

    @property
    def facebook_url(self) -> Optional[str]:
        return f"https://facebook.com/{self.facebook_id}" if self.facebook_id else None

    @property
    def instagram_url(self) -> Optional[str]:
        return f"https://instagram.com/{self.instagram_id}" if self.instagram_id else None

    @property
    def twitter_url(self) -> Optional[str]:
        return f"https://twitter.com/{self.twitter_id}" if self.twitter_id else None

    @property
    def tiktok_url(self) -> Optional[str]:
        return f"https://www.tiktok.com/@{self.tiktok_id}" if self.tiktok_id else None

    @property
    def youtube_url(self) -> Optional[str]:
        return f"https://www.youtube.com/@{self.youtube_id}" if self.youtube_id else None

    @property
    def wikidata_url(self) -> Optional[str]:
        return f"https://www.wikidata.org/wiki/{self.wikidata_id}" if self.wikidata_id else None

    def freebase_url(self, media_type: MediaType) -> Optional[str]:
        raise NotImplementedError()

    def tvdb_url(self, media_type: MediaType) -> Optional[str]:
        raise NotImplementedError()

    def tvrage_url(self, media_type: MediaType) -> Optional[str]:
        raise NotImplementedError()



from dataclasses import dataclass
from datetime import date
from typing import List, Optional

from themoviedb.schemas._enums import MediaType, SizeType


@dataclass
class Role:
    credit_id: Optional[str] = None
    character: Optional[str] = None
    episode_count: Optional[int] = None


@dataclass
class Job:
    credit_id: Optional[str] = None
    job: Optional[str] = None
    episode_count: Optional[int] = None


@dataclass
class Cast:
    id: int
    adult: Optional[bool] = None
    gender: Optional[int] = None
    known_for_department: Optional[str] = None
    name: Optional[str] = None
    original_name: Optional[str] = None
    popularity: Optional[float] = None
    profile_path: Optional[str] = None
    cast_id: Optional[int] = None
    character: Optional[str] = None
    credit_id: Optional[str] = None
    order: Optional[int] = None
    total_episode_count: Optional[int] = None
    roles: Optional[List[Role]] = None

    def __str__(self) -> str:
        return self.name or self.original_name

    def profile_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.profile_path}" if self.profile_path else None


@dataclass
class Crew:
    id: int
    adult: Optional[bool] = None
    gender: Optional[int] = None
    known_for_department: Optional[str] = None
    name: Optional[str] = None
    original_name: Optional[str] = None
    popularity: Optional[float] = None
    profile_path: Optional[str] = None
    credit_id: Optional[str] = None
    department: Optional[str] = None
    job: Optional[str] = None
    total_episode_count: Optional[int] = None
    jobs: Optional[List[Job]] = None
    order: Optional[int] = None

    def __str__(self) -> str:
        return self.name or self.original_name

    def profile_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.profile_path}" if self.profile_path else None


@dataclass
class GuestStars:
    id: Optional[int] = None
    character_name: Optional[str] = None
    credit_id: Optional[str] = None
    order: Optional[int] = None
    adult: Optional[bool] = None
    gender: Optional[int] = None
    known_for_department: Optional[str] = None
    name: Optional[str] = None
    original_name: Optional[str] = None
    popularity: Optional[float] = None
    profile_path: Optional[str] = None


@dataclass
class Credits:
    id: Optional[int] = None
    cast: Optional[List[Cast]] = None
    crew: Optional[List[Crew]] = None
    guest_stars: Optional[List[GuestStars]] = None


@dataclass
class CastMovie:
    id: Optional[int] = None
    character: Optional[str] = None
    credit_id: Optional[str] = None
    release_date: Optional[date] = None
    vote_count: Optional[int] = None
    video: Optional[bool] = None
    adult: Optional[bool] = None
    vote_average: Optional[float] = None
    title: Optional[str] = None
    genre_ids: Optional[List[int]] = None
    original_language: Optional[str] = None
    original_title: Optional[str] = None
    popularity: Optional[float] = None
    backdrop_path: Optional[str] = None
    overview: Optional[str] = None
    poster_path: Optional[str] = None
    order: Optional[int] = None

    def __str__(self) -> str:
        return self.title

    def backdrop_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.backdrop_path}" if self.backdrop_path else None

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class CrewMovie:
    id: Optional[int] = None
    department: Optional[str] = None
    original_language: Optional[str] = None
    original_title: Optional[str] = None
    job: Optional[str] = None
    overview: Optional[str] = None
    vote_count: Optional[int] = None
    video: Optional[bool] = None
    poster_path: Optional[str] = None
    backdrop_path: Optional[str] = None
    title: Optional[str] = None
    popularity: Optional[float] = None
    genre_ids: Optional[List[int]] = None
    vote_average: Optional[float] = None
    adult: Optional[bool] = None
    release_date: Optional[date] = None
    credit_id: Optional[str] = None
    order: Optional[int] = None

    def __str__(self) -> str:
        return self.title

    def backdrop_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.backdrop_path}" if self.backdrop_path else None

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class CreditsMovie:
    id: Optional[int] = None
    cast: Optional[List[CastMovie]] = None
    crew: Optional[List[CrewMovie]] = None


@dataclass
class CastTV:
    id: Optional[int] = None
    credit_id: Optional[str] = None
    original_name: Optional[str] = None
    genre_ids: Optional[List[int]] = None
    character: Optional[str] = None
    name: Optional[str] = None
    poster_path: Optional[str] = None
    vote_count: Optional[int] = None
    vote_average: Optional[float] = None
    popularity: Optional[float] = None
    episode_count: Optional[int] = None
    original_language: Optional[str] = None
    adult: Optional[bool] = None
    first_air_date: Optional[date] = None
    backdrop_path: Optional[str] = None
    overview: Optional[str] = None
    origin_country: Optional[List[str]] = None
    order: Optional[int] = None

    def __str__(self) -> str:
        return self.name or self.original_name

    def backdrop_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.backdrop_path}" if self.backdrop_path else None

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class CrewTV:
    id: Optional[int] = None
    department: Optional[str] = None
    original_language: Optional[str] = None
    episode_count: Optional[int] = None
    job: Optional[str] = None
    overview: Optional[str] = None
    origin_country: Optional[List[str]] = None
    original_name: Optional[str] = None
    genre_ids: Optional[List[int]] = None
    name: Optional[str] = None
    first_air_date: Optional[date] = None
    backdrop_path: Optional[str] = None
    popularity: Optional[float] = None
    adult: Optional[bool] = None
    vote_count: Optional[int] = None
    vote_average: Optional[float] = None
    poster_path: Optional[str] = None
    credit_id: Optional[str] = None
    order: Optional[int] = None

    def __str__(self) -> str:
        return self.name or self.original_name

    def backdrop_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.backdrop_path}" if self.backdrop_path else None

    def poster_url(self, size: Optional[SizeType] = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size}{self.poster_path}" if self.poster_path else None


@dataclass
class CreditsTV:
    id: Optional[int] = None
    cast: Optional[List[CastTV]] = None
    crew: Optional[List[CrewTV]] = None


@dataclass
class CastCombined(CastMovie, CastTV):
    media_type: Optional[MediaType] = None


@dataclass
class CrewCombined(CrewMovie, CrewTV):
    media_type: Optional[MediaType] = None


@dataclass
class CreditsCombined:
    id: Optional[int] = None
    cast: Optional[List[CastCombined]] = None
    crew: Optional[List[CrewCombined]] = None



from themoviedb.routes_sync import (
    Base,
    Authentication,
    Certifications,
    Collection,
    Company,
    Credit,
    Discover,
    Episode,
    EpisodeGroup,
    Find,
    Genres,
    Guest,
    Keyword,
    Movie,
    Movies,
    Network,
    People,
    Person,
    Review,
    Search,
    Season,
    Trending,
    TV,
    TVs,
    WatchProviders,
)


class TMDb(Base):
    """TMDb class.

    This class provides methods for accessing various endpoints of the TMDb API.
    Each method returns an instance of a corresponding model class, which can be used
    to retrieve information about a specific resource.
    """

    def authentication(self) -> Authentication:
        """Get model object for `themoviedb.Authentication` resource."""
        return Authentication(key=self.key, session=self.session, language=self.language, region=self.region)

    def certifications(self) -> Certifications:
        """Get model object for `themoviedb.Certifications` resource."""
        return Certifications(key=self.key, session=self.session, language=self.language, region=self.region)

    def collection(self, collection_id: int) -> Collection:
        """Get model object for `themoviedb.Collection` resource."""
        return Collection(collection_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def company(self, company_id: int) -> Company:
        """Get model object for `themoviedb.Company` resource."""
        return Company(company_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def credit(self, credit_id: int) -> Credit:
        """Get model object for `themoviedb.Credit` resource."""
        return Credit(credit_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def discover(self) -> Discover:
        """Get model object for `themoviedb.Discover` resource."""
        return Discover(key=self.key, session=self.session, language=self.language, region=self.region)

    def episode(self, tv_id: int, season_id: int, episode_id: int) -> Episode:
        """Get model object for `themoviedb.Episode` resource."""
        return Episode(tv_id, season_id, episode_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def episode_group(self, episode_group_id: int) -> EpisodeGroup:
        """Get model object for `themoviedb.EpisodeGroup` resource."""
        return EpisodeGroup(episode_group_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def find(self) -> Find:
        """Get model object for `themoviedb.Find` resource."""
        return Find(key=self.key, session=self.session, language=self.language, region=self.region)

    def genres(self) -> Genres:
        """Get model object for `themoviedb.Genres` resource."""
        return Genres(key=self.key, session=self.session, language=self.language, region=self.region)

    def guest(self, guest_session_id: str) -> Guest:
        """Get model object for `themoviedb.Guest` resource."""
        return Guest(guest_session_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def keyword(self, keyword_id: int) -> Keyword:
        """Get model object for `themoviedb.Keyword` resource."""
        return Keyword(keyword_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def movie(self, movie_id: int) -> Movie:
        """Get model object for `themoviedb.Movie` resource."""
        return Movie(movie_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def movies(self) -> Movies:
        """Get model object for `themoviedb.Movies` resource."""
        return Movies(key=self.key, session=self.session, language=self.language, region=self.region)

    def network(self, network_id: int) -> Network:
        """Get model object for `themoviedb.Network` resource."""
        return Network(network_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def people(self) -> People:
        """Get model object for `themoviedb.People` resource."""
        return People(key=self.key, session=self.session, language=self.language, region=self.region)

    def person(self, person_id: int) -> Person:
        """Get model object for `themoviedb.Person` resource."""
        return Person(person_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def review(self, review_id: int) -> Review:
        """Get model object for `themoviedb.Review` resource."""
        return Review(review_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def search(self) -> Search:
        """Get model object for `themoviedb.Search` resource."""
        return Search(key=self.key, session=self.session, language=self.language, region=self.region)

    def season(self, tv_id: int, season_id: int) -> Season:
        """Get model object for `themoviedb.Season` resource."""
        return Season(tv_id, season_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def trending(self) -> Trending:
        """Get model object for `themoviedb.Trending` resource."""
        return Trending(key=self.key, session=self.session, language=self.language, region=self.region)

    def tv(self, tv_id: int) -> TV:
        """Get model object for `themoviedb.TV` resource."""
        return TV(tv_id, key=self.key, session=self.session, language=self.language, region=self.region)

    def tvs(self) -> TVs:
        """Get model object for `themoviedb.TVs` resource."""
        return TVs(key=self.key, session=self.session, language=self.language, region=self.region)

    def watch_providers(self) -> WatchProviders:
        """Get model object for `themoviedb.WatchProviders` resource."""
        return WatchProviders(key=self.key, session=self.session, language=self.language, region=self.region)



from dataclasses import dataclass
from typing import Optional


@dataclass
class Language:
    english_name: Optional[str] = None
    iso_639_1: Optional[str] = None
    name: Optional[str] = None

    def __str__(self) -> str:
        return self.name
